Fill

Struct Fill 

pub struct Fill {
Show 14 fields pub entry_id: String, pub trade_id: String, pub order_id: String, pub trade_time: DateTime, pub trade_type: TradeType, pub price: String, pub size: String, pub commission: String, pub product_id: String, pub sequence_timestamp: DateTime, pub liquidity_indicator: LiquidityIndicator, pub size_in_quote: bool, pub user_id: String, pub side: OrderSide,
}
Structure representing CB’s response to a fill request

Fields

entry_id: String

Unique identifier for the fill.

trade_id: String

ID of the fill – unique for all FILL trade_types but not unique for adjusted fills.

order_id: String

ID of the order the fill belongs to.

trade_time: DateTime

Time at which this fill was completed.

trade_type: TradeType

String denoting what type of fill this is. Regular fills have the value FILL. Adjusted fills have possible values REVERSAL, CORRECTION, SYNTHETIC.

price: String

Price the fill was posted at.

size: String

Amount of order that was transacted at this fill.

commission: String

Fee amount for fill.

product_id: String

The product this order was created for.

sequence_timestamp: DateTime

Time at which this fill was posted.

liquidity_indicator: LiquidityIndicator

Possible values: [UNKNOWN_LIQUIDITY_INDICATOR, MAKER, TAKER]

size_in_quote: bool

Whether the order was placed with quote currency.

user_id: String

User that placed the order the fill belongs to.

side: OrderSide

Possible values: [UNKNOWN_ORDER_SIDE, BUY, SELL]