pub struct Limit {
pub base_size: BigDecimal,
pub limit_price: BigDecimal,
pub end_time: Option<DateTime>,
pub post_only: Option<bool>,
}Expand description
Structure representing Coinbase’s limit order structure
end_time is only used for gtd orders, not gtc
Fields
base_size: BigDecimalAmount of base currency to spend on order
limit_price: BigDecimalCeiling price for which the order should get filled
end_time: Option<DateTime>Time at which the order should be cancelled if it’s not filled.
post_only: Option<bool>Post only limit order