Order

Struct Order 

pub struct Order {
Show 30 fields pub order_id: String, pub product_id: String, pub user_id: String, pub order_configuration: OrderConfiguration, pub side: Side, pub client_order_id: String, pub status: Status, pub time_in_force: TimeInForce, pub created_time: DateTime, pub completion_percentage: String, pub filled_size: String, pub average_filled_price: String, pub fee: String, pub number_of_fills: String, pub filled_value: String, pub pending_cancel: bool, pub size_in_quote: bool, pub total_fees: String, pub size_inclusive_of_fees: bool, pub total_value_after_fees: String, pub trigger_status: TriggerStatus, pub order_type: OrderType, pub reject_reason: RejectReason, pub settled: bool, pub product_type: ProductType, pub reject_message: Option<String>, pub cancel_message: Option<String>, pub order_placement_source: OrderPlacementSource, pub outstanding_hold_amount: String, pub is_liquidation: bool,
}
Structure representing an order response

Fields

order_id: String

The unique id for this order

product_id: String

The product this order was created for e.g. ‘BTC-USD’

user_id: String

The id of the User owning this Order

order_configuration: OrderConfigurationside: Side

Possible values: [UNKNOWN_ORDER_SIDE, BUY, SELL]

client_order_id: String

Client specified ID of order.

status: Status

Possible values: [OPEN, FILLED, CANCELLED, EXPIRED, FAILED, UNKNOWN_ORDER_STATUS]

time_in_force: TimeInForce

Possible values: [UNKNOWN_TIME_IN_FORCE, GOOD_UNTIL_DATE_TIME, GOOD_UNTIL_CANCELLED, IMMEDIATE_OR_CANCEL, FILL_OR_KILL]

created_time: DateTime

Timestamp for when the order was created

completion_percentage: String

The percent of total order amount that has been filled

filled_size: String

The portion (in base currency) of total order amount that has been filled

average_filled_price: String

The average of all prices of fills for this order

fee: String

Commission amount

number_of_fills: String

Number of fills that have been posted for this order

filled_value: String

The portion (in quote current) of total order amount that has been filled

pending_cancel: bool

Whether a cancel request has been initiated for the order, and not yet completed

size_in_quote: bool

Whether the order was placed with quote currency

total_fees: String

The total fees for the order

size_inclusive_of_fees: bool

Whether the order size includes fees

total_value_after_fees: String

derived field: filled_value + total_fees for buy orders and filled_value - total_fees for sell orders.

trigger_status: TriggerStatus

Possible values: [UNKNOWN_TRIGGER_STATUS, INVALID_ORDER_TYPE, STOP_PENDING, STOP_TRIGGERED]

order_type: OrderType

Possible values: [UNKNOWN_ORDER_TYPE, MARKET, LIMIT, STOP, STOP_LIMIT]

reject_reason: RejectReason

Possible values: REJECT_REASON_UNSPECIFIED

settled: boolproduct_type: ProductType

Possible values: [SPOT, FUTURE]

reject_message: Option<String>

Message stating why the order was rejected.

cancel_message: Option<String>

Message stating why the order was canceled.

order_placement_source: OrderPlacementSource

Possible values: [RETAIL_SIMPLE, RETAIL_ADVANCED]

outstanding_hold_amount: Stringis_liquidation: bool

True if order is of liquidation type.