OrderConfiguration

Struct OrderConfiguration 

pub struct OrderConfiguration {
    pub market_market_ioc: Option<Market>,
    pub limit_limit_gtc: Option<Limit>,
    pub limit_limit_gtd: Option<Limit>,
    pub stop_limit_stop_limit_gtc: Option<StopLimit>,
    pub stop_limit_stop_limit_gtd: Option<StopLimit>,
}
Structure representing Coinbase’s order configuration structure

Option can nornally be not None for one of the item. It is not an enum to be able to deserialize the response

Fields

market_market_ioc: Option<Market>limit_limit_gtc: Option<Limit>limit_limit_gtd: Option<Limit>stop_limit_stop_limit_gtc: Option<StopLimit>stop_limit_stop_limit_gtd: Option<StopLimit>