pub struct StopLimit {
pub base_size: BigDecimal,
pub limit_price: BigDecimal,
pub stop_price: BigDecimal,
pub stop_direction: StopDirection,
pub end_time: Option<DateTime>,
}Expand description
Structure representing Coinbase’s stop-limit order structure
end_time is only used for gtd orders, not gtc
Fields
base_size: BigDecimalAmount of base currency to spend on order
limit_price: BigDecimalCeiling price for which the order should get filled
stop_price: BigDecimalPrice at which the order should trigger - if stop direction is Up, then the order will trigger when the last trade price goes above this, otherwise order will trigger when last trade price goes below this price.
stop_direction: StopDirectionend_time: Option<DateTime>Trait Implementations
Sourceimpl<'de> Deserialize<'de> for StopLimit
impl<'de> Deserialize<'de> for StopLimit
Sourcefn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
Deserialize this value from the given Serde deserializer. Read more
impl Eq for StopLimit
impl StructuralPartialEq for StopLimit