pub struct Order {Show 30 fields
pub order_id: String,
pub product_id: String,
pub user_id: String,
pub order_configuration: OrderConfiguration,
pub side: Side,
pub client_order_id: String,
pub status: Status,
pub time_in_force: TimeInForce,
pub created_time: DateTime,
pub completion_percentage: String,
pub filled_size: String,
pub average_filled_price: String,
pub fee: String,
pub number_of_fills: String,
pub filled_value: String,
pub pending_cancel: bool,
pub size_in_quote: bool,
pub total_fees: String,
pub size_inclusive_of_fees: bool,
pub total_value_after_fees: String,
pub trigger_status: TriggerStatus,
pub order_type: OrderType,
pub reject_reason: RejectReason,
pub settled: bool,
pub product_type: ProductType,
pub reject_message: Option<String>,
pub cancel_message: Option<String>,
pub order_placement_source: OrderPlacementSource,
pub outstanding_hold_amount: String,
pub is_liquidation: bool,
}Expand description
Structure representing an order response
Fields
order_id: StringThe unique id for this order
product_id: StringThe product this order was created for e.g. ‘BTC-USD’
user_id: StringThe id of the User owning this Order
order_configuration: OrderConfigurationside: SidePossible values: [UNKNOWN_ORDER_SIDE, BUY, SELL]
client_order_id: StringClient specified ID of order.
status: StatusPossible values: [OPEN, FILLED, CANCELLED, EXPIRED, FAILED, UNKNOWN_ORDER_STATUS]
time_in_force: TimeInForcePossible values: [UNKNOWN_TIME_IN_FORCE, GOOD_UNTIL_DATE_TIME, GOOD_UNTIL_CANCELLED, IMMEDIATE_OR_CANCEL, FILL_OR_KILL]
created_time: DateTimeTimestamp for when the order was created
completion_percentage: StringThe percent of total order amount that has been filled
filled_size: StringThe portion (in base currency) of total order amount that has been filled
average_filled_price: StringThe average of all prices of fills for this order
fee: StringCommission amount
number_of_fills: StringNumber of fills that have been posted for this order
filled_value: StringThe portion (in quote current) of total order amount that has been filled
pending_cancel: boolWhether a cancel request has been initiated for the order, and not yet completed
size_in_quote: boolWhether the order was placed with quote currency
total_fees: StringThe total fees for the order
size_inclusive_of_fees: boolWhether the order size includes fees
total_value_after_fees: Stringderived field: filled_value + total_fees for buy orders and filled_value - total_fees for sell orders.
trigger_status: TriggerStatusPossible values: [UNKNOWN_TRIGGER_STATUS, INVALID_ORDER_TYPE, STOP_PENDING, STOP_TRIGGERED]
order_type: OrderTypePossible values: [UNKNOWN_ORDER_TYPE, MARKET, LIMIT, STOP, STOP_LIMIT]
reject_reason: RejectReasonPossible values: REJECT_REASON_UNSPECIFIED
settled: boolproduct_type: ProductTypePossible values: [SPOT, FUTURE]
reject_message: Option<String>Message stating why the order was rejected.
cancel_message: Option<String>Message stating why the order was canceled.
order_placement_source: OrderPlacementSourcePossible values: [RETAIL_SIMPLE, RETAIL_ADVANCED]
outstanding_hold_amount: Stringis_liquidation: boolTrue if order is of liquidation type.